@@ -32,7 +32,6 @@ int record_hash_table[HASHTBSIZE][RECORDMAX];	// for hash record database
 int record_pt = 1;		// current index of record database
 int counter[31];		// counter str_set,str_dec ... 
 int catch_data[CTRLSTK][2][THREADSIZE];	//catch tag,sp,wp
-char bridge[BUFSIZE];		// for string_term/2 and parallel buffer
 char transfer[BUFSIZE];		// buffer for dp_transfer
 token stok = { GO, OTHER };
 
@@ -129,7 +128,7 @@ int prefix_flag = 0;		//for parser 0=not prefix, 1=prefix
 int syntax_flag = YES;		//syntaxerrors/2 YES=normal. NO=ignore syntax-errors
 int fileerr_flag = YES;		//fileerrors/2 YES=normal. NO=ignore file-errors
 int exist_flag = YES;		//existerrors/2 YES=normal, NO=ignore existance_errors
-int bridge_flag = 0;		//for string_term/2 0=normal, 1=readparse from bridge
+int dialog_flag = 0;		//for string_term/2 0=normal, 1=readparse from input output buffer
 int ctrl_c_flag = 0;		//for ctrl_c  to stop prove
 int init_flag = 1;		//for halt
 int script_flag = 0;		// script mode, 0=not scriplt-mode, 1=script-mode.
@@ -141,9 +140,7 @@ int thread_flag = 0;		/* when invoke as multi thread, flag is 1 */
 int child_flag = 0;		/* when invoke as network child, flag is 1 */
 int connect_flag = 0;		/* when child listen, connect_flag is 1 */
 int receiver_exit_flag = 0;	/* TO exit child TCP/IP receiver */
-int child_busy_flag = 0;	/* while executing in child, child_busy_flag is 1 */
 int parent_flag = 0;		/* while comunicating child, parent_flag = 1 */
-int pause_flag = 0;		/* while pause in child, pause_flag = 1 */
 int shutdown_flag = 0;		/* when receive dp_close, shutdown_flag = 1 */
 int active_thread = 0;		/* for mt_and/1 mt_or/1 */
 int dynamic_flag = 0;		/* for dynamic predicate. while assertz dynamic flag = 1 */
@@ -160,13 +157,23 @@ int output_stream;
 int error_stream;
 
 /* -----distributed parallel & TCPIP------*/
+pthread_mutex_t mutex2;
+pthread_cond_t md_cond; 
+int child_buffer_ready;
 int parent_sockfd[2];
 int child_sockfd[PARASIZE];
 socklen_t parent_len;
 struct sockaddr_in parent_addr, child_addr[PARASIZE];
 int child_num;
-pthread_t receiver_thread;
-int child_result[PARASIZE];
+int child_id;
+pthread_t preceiver_thread[PARASIZE];
+pthread_t creceiver_thread;
+char input_buffer[BUFSIZE];	
+char output_buffer[BUFSIZE];
+char parent_buffer[BUFSIZE][PARASIZE];
+char child_buffer[BUFSIZE]; 
+int child_buffer_pos;
+int child_buffer_end;	
 
 /* multi thread */
 pthread_mutex_t mutex;
@@ -257,7 +264,6 @@ int main(int argc, char *argv[])
     int ch, input;
     char *path;
     struct winsize w;
-    FILE *fp;
 
     signal(SIGINT, reset);
     init_cell();
@@ -304,14 +310,9 @@ int main(int argc, char *argv[])
 	    exit(EXIT_SUCCESS);
 	case 'n':
 	    printf("N-Prolog runs with network mode.\n");
-	    fp = fopen("network.pl", "r");
-	    if (fp != NULL) {
-		fclose(fp);
-		b_consult(list1(makeconst("network.pl")), NIL, 0);
-	    }
 	    child_flag = 1;
 	    init_parent();
-	    init_receiver();
+	    init_creceiver();
 	    break;
 	default:
 	    usage();
@@ -346,6 +347,12 @@ int main(int argc, char *argv[])
 		output_stream = standard_output;
 		error_stream = standard_error;
 		init_repl();
+		pthread_mutex_lock(&mutex2);
+		while (!child_buffer_ready) {
+            pthread_cond_wait(&md_cond, &mutex2);
+        }
+		child_buffer_ready = 0;
+		pthread_mutex_unlock(&mutex2);
 		input =
 		    variable_to_call(convert_to_variable
 				     (str_to_pred(receive_from_parent()),
@@ -354,13 +361,11 @@ int main(int argc, char *argv[])
 		sprint(input);
 		printf("\n");
 		fflush(stdout);
-		child_busy_flag = 1;
 		query(input, 0);
-		child_busy_flag = 0;
-		printf("Send to parent ");
-		printf("\n");
+		printf("Send to parent %s\n", output_buffer);
 		fflush(stdout);
-	    }
+		memset(output_buffer, 0, sizeof(output_buffer));
+		}
     } else if (ret == 1) {
 	ret = 0;
 	goto repl;
@@ -369,7 +374,7 @@ int main(int argc, char *argv[])
 	    printf("Shutting down the system...\n");
 	    int ret = system("sudo shutdown now");
 	    if (ret == -1)
-		exception(SYSTEM_ERR, makestr("dp_close shatdown"), NIL,
+		exception(SYSTEM_ERR, makestr("dp_halt shatdown"), NIL,
 			  0);
 	}
 	return 0;
@@ -481,7 +486,7 @@ void query(int x, int th)
 	print(res);
 	printf("\n");
     } else {
-	bridge_flag = 1;
+	dialog_flag = 1;
 	if (res == YES) {
 	    printstr("dp_countup(");
 	    printint(proof[th]);
@@ -493,7 +498,7 @@ void query(int x, int th)
 	    printstr("),");
 	    printstr("fail.\n");
 	}
-	bridge_flag = 0;
+	dialog_flag = 0;
 	send_to_parent_buffer();
     }
     return;
@@ -619,21 +624,18 @@ int prove(int goal, int bindings, int rest, int th)
 
     proof[th]++;
     if (ctrl_c_flag) {
-	printf("ctrl+C\n");
-	longjmp(buf, 1);
-    }
-
-    if (pause_flag) {
-	printf("pause\n");
-	while (1) {
-	    sleep(1);
-	    if (!pause_flag)
-		break;
+	if (!child_flag) {
+	    printf("ctrl+C\n");
+	    longjmp(buf, 1);
+	} else {
+	    send_to_parent(makeconst("ctrl"));
+	    printf("ctrl+C\n");fflush(stdout);
+	    fflush(stdout);
+	    longjmp(buf, 1);
 	}
-	printf("resume\n");
     }
 
-
+    
     if (nest > 40000)
 	exception(RESOURCE_ERR, NIL, makestr("prove recursion over max"),
 		  th);
@@ -1296,38 +1298,38 @@ void print(int addr)
     char str1[STRSIZE];
 
     if (IS_ALPHA(addr)) {
-	if (!bridge_flag)
+	if (!dialog_flag)
 	    fprintf(GET_PORT(output_stream), "v_%d", addr - CELLSIZE);
 	else {
 	    sprintf(str1, "v_%d", addr - CELLSIZE);
-	    strcat(bridge, str1);
+	    strcat(output_buffer, str1);
 	}
 	return;
     }
     switch (GET_TAG(addr)) {
     case INTN:
-	if (!bridge_flag)
+	if (!dialog_flag)
 	    fprintf(GET_PORT(output_stream), "%d", GET_INT(addr));
 	else {
 	    sprintf(str1, "%d", GET_INT(addr));
-	    strcat(bridge, str1);
+	    strcat(output_buffer, str1);
 	}
 	break;
     case FLTN:
 	x = GET_FLT(addr);
 	if (x - (int) x != 0.0) {
-	    if (!bridge_flag)
+	    if (!dialog_flag)
 		fprintf(GET_PORT(output_stream), "%g", x);
 	    else {
 		sprintf(str1, "%g", x);
-		strcat(bridge, str1);
+		strcat(output_buffer, str1);
 	    }
 	} else {
-	    if (!bridge_flag)
+	    if (!dialog_flag)
 		fprintf(GET_PORT(output_stream), "%0.1f", x);
 	    else {
 		sprintf(str1, "%0.1f", x);
-		strcat(bridge, str1);
+		strcat(output_buffer, str1);
 	    }
 	}
 	break;
@@ -1341,11 +1343,11 @@ void print(int addr)
 	if (addr == NIL) {
 	    printstr("[]");
 	} else if (variablep(addr)) {
-	    if (!bridge_flag)
+	    if (!dialog_flag)
 		fprintf(GET_PORT(output_stream), "%s", GET_NAME(addr));
 	    else {
 		sprintf(str1, "%s", GET_NAME(addr));
-		strcat(bridge, str1);
+		strcat(output_buffer, str1);
 	    }
 	} else if (quoted_flag && atom_quote_p(addr))
 	    print_quoted(addr);
@@ -1428,25 +1430,25 @@ void print(int addr)
 	} else if (addr == standard_error) {
 	    printstr("<stderr>");
 	} else {
-	    if (!bridge_flag)
+	    if (!dialog_flag)
 		fprintf(GET_PORT(output_stream), "<stream%d>", addr);
 	    else {
 		sprintf(str1, "<stream%d>", addr);
-		strcat(bridge, str1);
+		strcat(output_buffer, str1);
 	    }
 	}
 	break;
     case SOCKET:
-	if (!bridge_flag)
+	if (!dialog_flag)
 	    fprintf(GET_PORT(output_stream), "<socket%d>", addr);
 	else {
 	    sprintf(str1, "<socket%d>", addr);
-	    strcat(bridge, str1);
+	    strcat(output_buffer, str1);
 	}
 	break;
     case STR:
 	if (quoted_flag) {
-	    if (!bridge_flag) {
+	    if (!dialog_flag) {
 		if (!string_flag)
 		    fprintf(GET_PORT(output_stream), "$%s$",
 			    GET_NAME(addr));
@@ -1458,7 +1460,7 @@ void print(int addr)
 		    sprintf(str1, "$%s$", GET_NAME(addr));
 		else
 		    sprintf(str1, "\"%s\"", GET_NAME(addr));
-		strcat(bridge, str1);
+		strcat(output_buffer, str1);
 	    }
 	} else {
 	    printstr(GET_NAME(addr));
@@ -1582,11 +1584,11 @@ void printc(char c)
 {
     char str1[STRSIZE];
 
-    if (!bridge_flag)
+    if (!dialog_flag)
 	fprintf(GET_PORT(output_stream), "%c", c);
     else {
 	sprintf(str1, "%c", c);
-	strcat(bridge, str1);
+	strcat(output_buffer, str1);
     }
 }
 
@@ -1594,11 +1596,11 @@ void printstr(char *s)
 {
     char str1[STRSIZE];
 
-    if (!bridge_flag)
+    if (!dialog_flag)
 	fprintf(GET_PORT(output_stream), "%s", s);
     else {
 	sprintf(str1, "%s", s);
-	strcat(bridge, str1);
+	strcat(output_buffer, str1);
     }
 }
 
@@ -1606,11 +1608,11 @@ void printint(int n)
 {
     char str1[STRSIZE];
 
-    if (!bridge_flag)
+    if (!dialog_flag)
 	fprintf(GET_PORT(output_stream), "%d", n);
     else {
 	sprintf(str1, "%d", n);
-	strcat(bridge, str1);
+	strcat(output_buffer, str1);
     }
 }
 
@@ -1619,11 +1621,11 @@ void printlong(int addr)
 {
     char str1[STRSIZE];
 
-    if (!bridge_flag)
+    if (!dialog_flag)
 	fprintf(GET_PORT(output_stream), "%lld", GET_LONG(addr));
     else {
 	sprintf(str1, "%lld", GET_LONG(addr));
-	strcat(bridge, str1);
+	strcat(output_buffer, str1);
     }
 }
 
@@ -1802,35 +1804,35 @@ void printlist_canonical(int addr)
     if (nullp(addr))
 	print(NIL);
     else if (!listp(cdr(addr))) {
-	if (!bridge_flag)
+	if (!dialog_flag)
 	    fprintf(GET_PORT(output_stream), "'.'(");
 	else
-	    strcat(bridge, "'.'(");
+	    strcat(output_buffer, "'.'(");
 	print(car(addr));
-	if (!bridge_flag)
+	if (!dialog_flag)
 	    fprintf(GET_PORT(output_stream), ",");
 	else
-	    strcat(bridge, ",");
+	    strcat(output_buffer, ",");
 	print(cdr(addr));
-	if (!bridge_flag)
+	if (!dialog_flag)
 	    fprintf(GET_PORT(output_stream), ")");
 	else
-	    strcat(bridge, ")");
+	    strcat(output_buffer, ")");
     } else {
-	if (!bridge_flag)
+	if (!dialog_flag)
 	    fprintf(GET_PORT(output_stream), "'.'(");
 	else
-	    strcat(bridge, "'.'(");
+	    strcat(output_buffer, "'.'(");
 	print(car(addr));
-	if (!bridge_flag)
+	if (!dialog_flag)
 	    fprintf(GET_PORT(output_stream), ",");
 	else
-	    strcat(bridge, ",");
+	    strcat(output_buffer, ",");
 	printlist_canonical(cdr(addr));
-	if (!bridge_flag)
+	if (!dialog_flag)
 	    fprintf(GET_PORT(output_stream), ")");
 	else
-	    strcat(bridge, ")");
+	    strcat(output_buffer, ")");
     }
 }
 
@@ -1838,18 +1840,18 @@ void printcurl(int addr)
 {
     arguments_flag = 0;		//(1,2,3) -> 1,2,3
     if (nullp(addr)) {
-	if (!bridge_flag)
+	if (!dialog_flag)
 	    fprintf(GET_PORT(output_stream), "}");
 	else
-	    strcat(bridge, "}");
+	    strcat(output_buffer, "}");
 	arguments_flag = 1;
     } else {
 	print(car(addr));
 	if (!nullp(cdr(addr))) {
-	    if (!bridge_flag)
+	    if (!dialog_flag)
 		fprintf(GET_PORT(output_stream), ",");
 	    else
-		strcat(bridge, ",");
+		strcat(output_buffer, ",");
 	}
 	printcurl(cdr(addr));
     }
@@ -1858,16 +1860,16 @@ void printcurl(int addr)
 void printtuple(int addr)
 {
     if (nullp(addr)) {
-	if (!bridge_flag)
+	if (!dialog_flag)
 	    fprintf(GET_PORT(output_stream), ")");
 	else
-	    strcat(bridge, ")");
+	    strcat(output_buffer, ")");
     } else {
 	print(car(addr));
 	if (!nullp(cdr(addr))) {
-	    if (!bridge_flag)
+	    if (!dialog_flag)
 		fprintf(GET_PORT(output_stream), ",");
-	    strcat(bridge, ",");
+	    strcat(output_buffer, ",");
 	}
 	printtuple(cdr(addr));
     }
@@ -1878,65 +1880,65 @@ void printclause(int addr)
 
     if (listing_flag) {
 	if (nullp(caddr(addr))) {	// e.g. :- foo(x).
-	    if (!bridge_flag)
+	    if (!dialog_flag)
 		fprintf(GET_PORT(output_stream), "(:- ");
 	    else
-		strcat(bridge, "(:- ");
+		strcat(output_buffer, "(:- ");
 	    print(cadr(addr));
-	    if (!bridge_flag)
+	    if (!dialog_flag)
 		fprintf(GET_PORT(output_stream), ")");
 	    else
-		strcat(bridge, ")");
+		strcat(output_buffer, ")");
 	} else {		// e.g. foo(x) :- boo(x).
 	    print(cadr(addr));
-	    if (!bridge_flag)
+	    if (!dialog_flag)
 		fprintf(GET_PORT(output_stream), " :-\n    ");
 	    else
-		strcat(bridge, " :-\n    ");
+		strcat(output_buffer, " :-\n    ");
 	    printbody(caddr(addr));
 	}
     } else {
 	if (!ignore_flag) {
 	    if (nullp(caddr(addr))) {	// e.g :- foo(x).
-		if (!bridge_flag)
+		if (!dialog_flag)
 		    fprintf(GET_PORT(output_stream), ":- ");
 		else
-		    strcat(bridge, ":- ");
+		    strcat(output_buffer, ":- ");
 		print(cadr(addr));
 	    } else {		//normal case  foo(X) :- bar(X).
 		print(cadr(addr));
-		if (!bridge_flag)
+		if (!dialog_flag)
 		    fprintf(GET_PORT(output_stream), ":-");
 		else
-		    strcat(bridge, ":-");
+		    strcat(output_buffer, ":-");
 		print(caddr(addr));
 	    }
 	} else {
 	    if (nullp(caddr(addr))) {	// e.g :- foo(x).
-		if (!bridge_flag)
+		if (!dialog_flag)
 		    fprintf(GET_PORT(output_stream), ":-(");
 		else
-		    strcat(bridge, ":-(");
+		    strcat(output_buffer, ":-(");
 		print(cadr(addr));
-		if (!bridge_flag)
+		if (!dialog_flag)
 		    fprintf(GET_PORT(output_stream), ")");
 		else
-		    strcat(bridge, ")");
+		    strcat(output_buffer, ")");
 	    } else {		// normal case foo(X) :- bar(X).
-		if (!bridge_flag)
+		if (!dialog_flag)
 		    fprintf(GET_PORT(output_stream), ":-(");
 		else
-		    strcat(bridge, ":-(");
+		    strcat(output_buffer, ":-(");
 		print(cadr(addr));
-		if (!bridge_flag)
+		if (!dialog_flag)
 		    fprintf(GET_PORT(output_stream), ",");
 		else
-		    strcat(bridge, ",");
+		    strcat(output_buffer, ",");
 		print(caddr(addr));
-		if (!bridge_flag)
+		if (!dialog_flag)
 		    fprintf(GET_PORT(output_stream), ")");
 		else
-		    strcat(bridge, ")");
+		    strcat(output_buffer, ")");
 	    }
 	}
     }
@@ -1948,22 +1950,22 @@ void printbody(int addr)
     if (operationp(addr) && car(addr) == AND) {
 	printbody(cadr(addr));
 	print(car(addr));
-	if (!bridge_flag)
+	if (!dialog_flag)
 	    fprintf(GET_PORT(output_stream), "\n    ");
-	strcat(bridge, "\n    ");
+	strcat(output_buffer, "\n    ");
 	printbody(caddr(addr));
     } else if (operationp(addr) && car(addr) == OR) {
-	if (!bridge_flag)
+	if (!dialog_flag)
 	    fprintf(GET_PORT(output_stream), "(");
 	else
-	    strcat(bridge, "(");
+	    strcat(output_buffer, "(");
 	printbody1(cadr(addr));
 	print(car(addr));
 	printbody1(caddr(addr));
-	if (!bridge_flag)
+	if (!dialog_flag)
 	    fprintf(GET_PORT(output_stream), ")");
 	else
-	    strcat(bridge, ")");
+	    strcat(output_buffer, ")");
     } else {
 	print(addr);
     }
