@@ -17,7 +17,7 @@ address
  5,000,001 - 30,000,000  working area 
 30,000,001 - 32,000,000  variant area
 */
-#define VERSION     4.55
+#define VERSION     4.61
 #define CELLSIZE    30000000  // if raspberry PI set smaller size.
 #define HEAPSIZE     5000000
 #define FREESIZE         500
@@ -38,7 +38,7 @@ address
 #define FUNCTION_NUMBER 22
 #define BUILTIN_NUMBER 220
 #define COMPILED_NUMBER 23
-#define EXTENDED_NUMBER 40
+#define EXTENDED_NUMBER 43
 #define NIL         0
 #define YES         2
 #define NO          4
@@ -231,8 +231,9 @@ extern int record_hash_table[HASHTBSIZE][RECORDMAX];
 extern int record_pt;       
 extern int counter[31]; 
 extern int catch_data[CTRLSTK][2][THREADSIZE]; 
-extern char bridge[BUFSIZE];
 extern char transfer[BUFSIZE];
+extern char input_buffer[BUFSIZE];
+extern char output_buffer[BUFSIZE];
 extern int variables[THREADSIZE];
 extern int variables_save[THREADSIZE];
 extern int end_of_file_answer;
@@ -330,13 +331,21 @@ enum Type {
     
 /* distributed parallel */
 #define PORT 5000 
+extern pthread_mutex_t mutex2;
+extern pthread_cond_t md_cond;
+extern int child_buffer_ready;
 extern int parent_sockfd[2];
 extern int child_sockfd[PARASIZE];
 extern socklen_t parent_len;
 extern struct sockaddr_in parent_addr, child_addr[PARASIZE];
 extern int child_num;
-extern pthread_t receiver_thread;
-extern int child_result[PARASIZE];
+extern int child_id;
+extern pthread_t preceiver_thread[PARASIZE];
+extern pthread_t creceiver_thread;
+extern char parent_buffer[BUFSIZE][PARASIZE];
+extern char child_buffer[BUFSIZE];
+extern int child_buffer_pos;
+extern int child_buffer_end;
 
 /* multi-thread */
 extern pthread_mutex_t mutex;
@@ -537,7 +546,7 @@ extern int prefix_flag;
 extern int syntax_flag;
 extern int fileerr_flag;
 extern int exist_flag;
-extern int bridge_flag;
+extern int dialog_flag;
 extern int ctrl_c_flag;
 extern int init_flag;
 extern int script_flag;
@@ -550,9 +559,7 @@ extern int thread_flag;
 extern int child_flag;
 extern int connect_flag;
 extern int receiver_exit_flag;
-extern int child_busy_flag;
 extern int parent_flag;
-extern int pause_flag;
 extern int shutdown_flag;
 extern int active_thread;
 extern int dynamic_flag;
@@ -843,6 +850,7 @@ int b_display(int arglist, int rest, int th);
 int b_dp_countup(int arglist, int rest, int th);
 int b_dp_create(int arglist, int rest, int th);
 int b_dp_close(int arglist, int rest, int th);
+int b_dp_halt(int arglist, int rest, int th);
 int b_dp_prove(int arglist ,int rest, int th);
 int b_dp_and(int arglist, int rest, int th);
 int b_dp_or(int arglist, int rest, int th);
@@ -850,12 +858,9 @@ int b_dp_transfer(int arglist, int rest, int th);
 int b_dp_receive(int arglist ,int rest, int th);
 int b_dp_compile(int arglist, int rest, int th);
 int b_dp_consult(int arglist, int rest, int th);
-int b_dp_report(int arglist, int rest, int th);
-int b_dp_parent(int arglist, int rest, int th);
-int b_dp_child(int arglist, int rest, int th);
-int b_dp_wait(int arglist, int rest, int th);
-int b_dp_pause(int arglist, int rest, int th);
-int b_dp_resume(int arglist, int rest, int th);
+int b_dp_reconsult(int arglist, int rest, int th);
+int b_dp_setid(int arglist, int rest, int th);
+int b_dp_senderr(int arglist, int rest, int th);
 int b_dup(int arglist, int rest, int th);
 int b_dynamic(int arglist, int rest, int th);
 int b_n_dynamic_predicate(int arglist, int rest, int th);
@@ -1330,9 +1335,6 @@ int readdouble(int th);
 int read_string_term(int flag);
 int receive_from_parent(void);
 int receive_from_child(int n);
-int receive_from_child_or(int n);
-int receive_from_child_or1(int n);
-int receive_from_child_or2(int n);
 int remove_cut(int x);
 int remove_duplicate(int x);
 int replace(int x, int lis);
@@ -1447,7 +1449,6 @@ void discard_trail(void);
 void discard_trail_n(int n);
 void dynamic_link(int x);
 void exit_para(void);
-//void error(int errnum, char *fun, int arg, int th);
 void exception(int errnum, int ind, int arg, int th);
 void errorcomp(int errnum, int name, int arg);
 void execute(int x);
@@ -1459,7 +1460,8 @@ void heapdump(int start, int end);
 void inctrace(int goal);
 void init_para(void);
 void init_parent(void);
-void init_receiver(void);
+void init_preceiver(int n);
+void init_creceiver(void);
 void init_repl(void);
 void init_builtin(void);
 void init_cell(void);
@@ -1503,10 +1505,13 @@ void query(int x, int th);
 void query_break(int x, int th);
 void recall_variant(int x);
 void release_variant(int x);
+void receive_from_parent_buffer(void);
 void reset(int i);
 void send_to_parent(int x);
 void send_to_parent_buffer(void);
+void send_to_parent_control(int code);
 void sned_to_child(int n, int x);
+void send_to_child_control(int n, int code);
 void set_sign(int x, int y);
 void sprint(int addr);
 void throw(int tag, int th);
