@@ -15,12 +15,12 @@ int pred_to_str(int x)
 {
     int res;
 
-    memset(bridge, '\0', sizeof(bridge));
-    bridge_flag = 1;
+    memset(output_buffer, '\0', sizeof(output_buffer));
+    dialog_flag = 1;
     print(x);
     printstr(".\n");
-    bridge_flag = 0;
-    res = makestr((char *) bridge);
+    dialog_flag = 0;
+    res = makestr((char *) output_buffer);
     return (res);
 }
 
@@ -28,13 +28,13 @@ int str_to_pred(int x)
 {
     int res;
 
-    bridge_flag = 1;
-    memset(bridge, '\0', sizeof(bridge));
-    strcpy(bridge, GET_NAME(x));
+    dialog_flag = 1;
+    memset(input_buffer, '\0', sizeof(input_buffer));
+    strcpy(input_buffer, GET_NAME(x));
     read_string_term(0);
     res = variable_to_call(readparse(0));
-    memset(bridge, 0, sizeof(bridge));
-    bridge_flag = 0;
+    memset(input_buffer, 0, sizeof(input_buffer));
+    dialog_flag = 0;
     return (res);
 }
 
@@ -133,320 +133,97 @@ void init_child(int n, int x)
 
 int receive_from_parent(void)
 {
-    int n;
-
-    if (!connect_flag) {
-	//wait conneting
-	listen(parent_sockfd[0], 5);
-	parent_len = sizeof(parent_addr);
-	connect_flag = 1;
-
-	// connection from parent
-	parent_sockfd[1] =
-	    accept(parent_sockfd[0], (struct sockaddr *) &parent_addr,
-		   &parent_len);
-	if (parent_sockfd[1] < 0) {
-	    exception(SYSTEM_ERR, makestr("receive from parent"), NIL, 0);
-	}
-    }
-    // read message from parent
-    memset(bridge, 0, sizeof(bridge));
-    n = read(parent_sockfd[1], bridge, sizeof(bridge) - 1);
-    if (n < 0) {
-	exception(SYSTEM_ERR, makestr("receive from parent"), NIL, 0);
-    }
-    return (makestr(bridge));
+	child_buffer_pos = 0;
+	child_buffer_end = 0;
+    return (makestr(child_buffer));
 }
 
 void send_to_parent(int x)
 {
-    int n;
+    int n,i;
 
     // send message to parent
-    memset(bridge, 0, sizeof(bridge));
-    strcpy(bridge, GET_NAME(x));
-    n = write(parent_sockfd[1], bridge, strlen(bridge));
-    memset(bridge, 0, sizeof(bridge));
+    memset(output_buffer, 0, sizeof(output_buffer));
+    strcpy(output_buffer, GET_NAME(x));
+	i = strlen(output_buffer);
+	output_buffer[i] = 0x16;
+	output_buffer[i+1] = 0;
+    n = write(parent_sockfd[1], output_buffer, i+1);
+    memset(output_buffer, 0, sizeof(output_buffer));
     if (n < 0) {
 	exception(SYSTEM_ERR, makestr("send to parent"), x, 0);
     }
+
 }
 
 void send_to_parent_buffer(void)
 {
-    int n;
+    int n,i;
 
-    n = write(parent_sockfd[1], bridge, strlen(bridge));
+	i = strlen(output_buffer);
+	output_buffer[i] = 0x16;
+	output_buffer[i+1] = 0;
+    n = write(parent_sockfd[1], output_buffer, i+1);
     if (n < 0) {
 	exception(SYSTEM_ERR, makestr("send to parent buffer"), NIL, 0);
     }
-    memset(bridge, 0, sizeof(bridge));
 }
 
+void send_to_parent_control(int code)
+{
+    int n;
+
+	memset(output_buffer,0,sizeof(output_buffer));
+	output_buffer[0] = code;
+	output_buffer[1] = 0x16;
+	output_buffer[2] = 0;
+    n = write(parent_sockfd[1], output_buffer, 2);
+    if (n < 0) {
+	exception(SYSTEM_ERR, makestr("send to parent code"), NIL, 0);
+    }
+}
+
+
 void send_to_child(int n, int x)
 {
-    int m;
+    int m,i;
 
-    memset(bridge, 0, sizeof(bridge));
-    strcpy(bridge, GET_NAME(x));
-    m = write(child_sockfd[n], bridge, strlen(bridge));
-    memset(bridge, 0, sizeof(bridge));
+    memset(output_buffer, 0, sizeof(output_buffer));
+    strcpy(output_buffer, GET_NAME(x));
+	i = strlen(output_buffer);
+	output_buffer[i] = 0x16;
+    m = write(child_sockfd[n], output_buffer, strlen(output_buffer));
+    memset(output_buffer, 0, sizeof(output_buffer));
     if (m < 0) {
 	exception(SYSTEM_ERR, makestr("send to child"), NIL, 0);
     }
 }
 
+// send one control code
+void send_to_child_control(int n, int code)
+{
+    int m;
+
+	memset(output_buffer,0,sizeof(output_buffer));
+	output_buffer[0] = code;
+	output_buffer[1] = 0x16;
+    m = write(child_sockfd[n], output_buffer, 2);
+    if (m < 0) {
+	exception(SYSTEM_ERR, makestr("send to child buffer"), NIL, 0);
+    }
+}
+
+
+
 int receive_from_child(int n)
 {
-    int m, i, j;
-    char sub_buffer[256], sub_buffer1[256 + 2];
-
-    // receive from child
-  reread:
-    memset(bridge, 0, sizeof(bridge));
-    m = read(child_sockfd[n], bridge, sizeof(bridge) - 1);
-    if (m < 0) {
-	exception(SYSTEM_ERR, makestr("receive from child"), makeint(n),
-		  0);
-    }
-
-  retry:
-    if (bridge[0] == '\x02') {
-	i = 0;
-	while (bridge[i + 1] != '\x03') {
-	    sub_buffer[i] = bridge[i + 1];
-	    i++;
-	}
-	if (!(child_flag && parent_flag)) {
-	    sub_buffer[i] = 0;
-	    printf("%s", sub_buffer);
-	} else {
-	    memset(sub_buffer1, 0, sizeof(sub_buffer1));
-	    sprintf(sub_buffer1, "\x02%s\x03", sub_buffer);
-	    send_to_parent(makestr(sub_buffer1));
-	}
-	j = 0;
-	i = i + 2;
-	while (bridge[j + i] != 0) {
-	    bridge[j] = bridge[j + i];
-	    j++;
-	}
-	bridge[j] = 0;
-	if (bridge[0] == 0)
-	    goto reread;
-	else
-	    goto retry;
-    } else if (bridge[0] == '\x15') {
-	if (!(child_flag && parent_flag)) {
-	    exception(SYSTEM_ERR, makestr("in child"), makeint(n), 0);
-	} else {
-	    memset(sub_buffer1, 0, sizeof(sub_buffer1));
-	    sub_buffer1[0] = '\x15';
-	    send_to_parent(makestr(sub_buffer1));
-	}
-    } else {
-	return (makestr(bridge));
-    }
-
-    return (0);
-}
-
-
-
-
-int receive_from_child_or(int n)
-{
-    int i, m, res;
-
-    //initialize -1 (not received)
-    for (i = 0; i < n; i++) {
-	child_result[i] = -1;
-    }
-    res = receive_from_child_or1(n);
-
-    // kill not received child
-    for (i = 0; i < n; i++) {
-	if (child_result[i] == -1) {
-	    // send child stop signal
-	    memset(bridge, 0, sizeof(bridge));
-	    bridge[0] = '\x11';
-	    m = write(child_sockfd[i], bridge, strlen(bridge));
-	    if (m < 0) {
-		exception(SYSTEM_ERR, makestr("receive from child"), NIL,
-			  0);
-	    }
-	    // receive result and ignore
-	    while ((m =
-		    read(child_sockfd[i], bridge,
-			 sizeof(bridge) - 1)) == 0) {
-	    }
-	}
-    }
-
-    return (res);
-}
-
-
-int receive_from_child_or1(int n)
-{
-    int m, i;
-
-    // receive from child
-    m = 0;
-  retry:
-    memset(bridge, 0, sizeof(bridge));
-    for (i = 0; i < n; i++) {
-	if (child_result[i] == -1) {
-	    m = read(child_sockfd[i], bridge, sizeof(bridge));
-	}
-	if (m < 0) {
-	    exception(SYSTEM_ERR, makestr("receive from child"),
-		      makeint(i), 0);
-	} else if (m > 0) {
-	    child_result[i] = receive_from_child_or2(i);
-	}
-    }
-
-    //if find true return it.
-    for (i = 0; i < n; i++) {
-	if (child_result[i] != -1
-	    && strcmp(GET_NAME(child_result[i]), "fail.") != 0)
-	    return (child_result[i]);
-    }
-
-
-    //if exist not received result, goto retry
-    for (i = 0; i < n; i++) {
-	if (child_result[i] == -1)
-	    goto retry;
-    }
-
-    // if not exist YES, return fail.
-    return (makeatom("fail", SYS));
-}
-
-int receive_from_child_or2(int n)
-{
-    char sub_buffer[256], sub_buffer1[256 + 2];
-    int i, j;
-
-  retry:
-    if (bridge[0] == '\x02') {
-	i = 0;
-	while (bridge[i + 1] != '\x03') {
-	    sub_buffer[i] = bridge[i + 1];
-	    i++;
-	}
-	if (!(child_flag && parent_flag)) {
-	    sub_buffer[i] = 0;
-	    printf("%s", sub_buffer);
-	} else {
-	    memset(sub_buffer1, 0, sizeof(sub_buffer1));
-	    sprintf(sub_buffer1, "\x02%s\x03", sub_buffer);
-	    send_to_parent(makestr(sub_buffer1));
-	}
-	j = 0;
-	i = i + 2;
-	while (bridge[j + i] != 0) {
-	    bridge[j] = bridge[j + i];
-	    j++;
-	}
-	bridge[j] = 0;
-	if (bridge[0] == 0)
-	    return (-1);
-	else
-	    goto retry;
-
-    } else if (bridge[0] == '\x15') {
-	if (!(child_flag && parent_flag)) {
-	    exception(SYSTEM_ERR, makestr("in child"), makeint(n), 0);
-	} else {
-	    memset(sub_buffer1, 0, sizeof(sub_buffer1));
-	    sub_buffer1[0] = '\x15';
-	    send_to_parent(makestr(sub_buffer1));
-	}
-    } else {
-	return (makestr(bridge));
-    }
-
-    return (0);
-}
-
-
-
-// Thread for child lisp receiver
-void *receiver(void *arg)
-{
-    int i;
-    char sub_buffer[256];
-
-    while (1) {
-	if (receiver_exit_flag)
-	    goto exit;
-
-	if (child_busy_flag) {
-	    receive_from_parent();
-	  retry:
-	    if (bridge[0] == '\x11') {
-		// child stop 
-		if (!(child_flag && parent_flag)) {
-		    memset(sub_buffer, 0, sizeof(sub_buffer));
-		    sub_buffer[0] = '\x11';
-		    for (i = 0; i < child_num; i++)
-			send_to_child(i, makestr(sub_buffer));
-		}
-		ctrl_c_flag = 1;
-	    } else if (bridge[0] == '\x12') {
-		// child pause 
-		if (!(child_flag && parent_flag)) {
-		    memset(sub_buffer, 0, sizeof(sub_buffer));
-		    sub_buffer[0] = '\x12';
-		    for (i = 0; i < child_num; i++)
-			send_to_child(i, makestr(sub_buffer));
-		}
-		pause_flag = 1;
-	    } else if (bridge[0] == '\x13') {
-		// child resume 
-		if (!(child_flag && parent_flag)) {
-		    memset(sub_buffer, 0, sizeof(sub_buffer));
-		    sub_buffer[0] = '\x11';
-		    for (i = 0; i < child_num; i++)
-			send_to_child(i, makestr(sub_buffer));
-		}
-		pause_flag = 0;
-	    }
-
-	    if (bridge[1] != 0) {
-		int i;
-		i = 0;
-		while (bridge[i + 1] != 0) {
-		    bridge[i] = bridge[i + 1];
-		    i++;
-		}
-		bridge[i] = 0;
-		goto retry;
-	    }
-
-	}
-
-    }
-
-  exit:
-    pthread_exit(NULL);
-}
-
-
-void init_receiver(void)
-{
-    // create child receiver thread 
-    pthread_create(&receiver_thread, NULL, receiver, NULL);
-
+	return (makestr(parent_buffer[n]));
 }
 
 
 int b_dp_create(int arglist, int rest, int th)
 {
-    int n, ind, arg1;
+    int n, ind, arg1, i, pred;
 
     n = length(arglist);
     ind = makeind("dp_create", n, th);
@@ -468,12 +245,51 @@ int b_dp_create(int arglist, int rest, int th)
 	    arg1 = cdr(arg1);
 	    child_num++;
 	}
+	init_preceiver(child_num);
+
+	for(i=0;i<child_num;i++){
+		pred = list2(makeatom("dp_setid",SYS),makeint(i));
+		send_to_child(i, pred_to_str(pred));
+	}
 	return (prove_all(rest, sp[th], th));
     }
     exception(ARITY_ERR, ind, arglist, th);
     return (NO);
 }
 
+int b_dp_setid(int arglist, int rest, int th)
+{
+	int n, arg1, ind;
+
+    n = length(arglist);
+    ind = makeind("dp_setid", n, th);
+    if (n == 1) {
+		arg1 = car(arglist);
+		child_id = GET_INT(arg1);
+		return (prove_all(rest, sp[th], th));
+	}
+	exception(ARITY_ERR, ind, arglist, th);
+    return (NO);
+}
+
+int b_dp_senderr(int arglist, int rest, int th)
+{
+	int n, arg1, ind;
+
+    n = length(arglist);
+    ind = makeind("dp_senderr", n, th); 
+    if (n == 1) {
+		arg1 = car(arglist);
+		
+		printf("occured an error in child %d\n",GET_INT(arg1));
+		fflush(stdout);
+		return (NO);
+	}
+	exception(ARITY_ERR, ind, arglist, th);
+    return (NO);
+}
+
+
 // close all distributed child 
 int b_dp_close(int arglist, int rest, int th)
 {
@@ -484,21 +300,48 @@ int b_dp_close(int arglist, int rest, int th)
     if (n == 0) {
 
 	if (parent_flag) {
-	    exp = makestr("dp_close.");
+	    exp = makeatom("dp_close",SYS);
 	    for (i = 0; i < child_num; i++) {
-		send_to_child(i, exp);
+		send_to_child(i, pred_to_str(exp));
 	    }
 	}
 
-	if (parent_flag) {
-	    for (i = 0; i < child_num; i++)
-		close(child_sockfd[i]);
-	}
+
 
 	if (child_flag) {
 	    printf("N-Prolog exit network mode.\n");
-	    close(parent_sockfd[0]);
-	    close(parent_sockfd[1]);
+	    longjmp(buf, 2);
+	}
+
+	child_num = 0;
+	parent_flag = 0;
+	receiver_exit_flag = 1;
+	return (prove_all(rest, sp[th], th));
+    }
+    exception(ARITY_ERR, ind, arglist, 0);
+    return (NO);
+
+}
+
+// close all distributed child and shutdown 
+int b_dp_halt(int arglist, int rest, int th)
+{
+    int n, ind, i, exp;
+
+    n = length(arglist);
+    ind = makeind("dp_halt", n, th);
+    if (n == 0) {
+
+	if (parent_flag) {
+	    exp = makeatom("dp_halt",SYS);
+	    for (i = 0; i < child_num; i++) {
+		send_to_child(i, pred_to_str(exp));
+	    }
+	}
+
+	
+	if (child_flag) {
+	    printf("N-Prolog exit network mode.\n");
 	    receiver_exit_flag = 1;
 	    shutdown_flag = 1;
 	    longjmp(buf, 2);
@@ -517,7 +360,7 @@ int b_dp_close(int arglist, int rest, int th)
 
 int b_dp_prove(int arglist, int rest, int th)
 {
-    int n, ind, arg1, arg2, res;
+    int n, ind, arg1, arg2, res ,i;
 
     n = length(arglist);
     ind = makeind("dp_prove", n, th);
@@ -529,12 +372,24 @@ int b_dp_prove(int arglist, int rest, int th)
 	if (!callablep(arg2))
 	    exception(NOT_CALLABLE, ind, arg2, th);
 
+	i = GET_INT(arg1);
+	memset(parent_buffer[i],0,sizeof(parent_buffer[i]));
 	send_to_child(GET_INT(arg1), pred_to_str(arg2));
+	while(parent_buffer[i][0] == 0){
+	usleep(1000);
+	if(ctrl_c_flag == 1){
+		send_to_child_control(GET_INT(arg1),0x11);
+		printf("ctrl+C\n");
+	    longjmp(buf, 1);
+	}
+	}
 	res =
-	    convert_to_variant(str_to_pred
-			       (receive_from_child(GET_INT(arg1))), th);
+	    convert_to_variant(str_to_pred(receive_from_child(i)), th);
+	
 	if (prove_all(res, sp[th], th) == YES)
-	    return (prove_all(rest, sp[th], th));
+		return (prove_all(rest, sp[th], th));
+	else
+	    return (NO);
     }
     exception(ARITY_ERR, ind, arglist, th);
     return (NO);
@@ -543,13 +398,14 @@ int b_dp_prove(int arglist, int rest, int th)
 // parent Prolog
 int b_dp_transfer(int arglist, int rest, int th)
 {
-    int n, ind, arg1, pred1, pred2, i, m;
+    int n, ind, arg1, pred1, i, m;
     FILE *file;
 
     n = length(arglist);
     ind = makeind("dp_transfer", n, th);
     if (n == 1) {
 	arg1 = car(arglist);
+	arg1 = makeatom(prolog_file_name(GET_NAME(arg1)), SIMP);
 	if (!atomp(arg1))
 	    exception(NOT_ATOM, ind, arg1, th);
 
@@ -573,7 +429,7 @@ int b_dp_transfer(int arglist, int rest, int th)
 		}
 	    }
 	    memset(transfer, 0, sizeof(transfer));
-	    transfer[0] = 0x15;
+	    transfer[0] = 0x16;
 	    m = write(child_sockfd[i], transfer, 1);
 	    if (m < 0) {
 		exception(SYSTEM_ERR, makestr("dp_transfer"), NIL, th);
@@ -582,11 +438,6 @@ int b_dp_transfer(int arglist, int rest, int th)
 	    fseek(file, 0, SEEK_SET);
 	}
 	fclose(file);
-	if (parent_flag) {
-	    pred2 = list2(makeatom("dp_transfer", SYS), arg1);
-	    for (i = 0; i < child_num; i++)
-		send_to_child(i, pred_to_str(pred2));
-	}
 	return (prove_all(rest, sp[th], th));
     }
     exception(ARITY_ERR, ind, arglist, th);
@@ -602,10 +453,8 @@ int b_dp_receive(int arglist, int rest, int th)
     n = length(arglist);
     ind = makeind("dp_recieve", n, th);
     if (n == 1) {
-	child_busy_flag = 0;
 	arg1 = car(arglist);
 
-
 	file = fopen(GET_NAME(arg1), "w");
 	if (!file) {
 	    exception(CANT_OPEN, ind, arg1, th);
@@ -614,14 +463,13 @@ int b_dp_receive(int arglist, int rest, int th)
 	int bytes_received;
 	while ((bytes_received =
 		read(parent_sockfd[1], transfer, sizeof(transfer))) > 0) {
-	    if (transfer[bytes_received - 1] == 0x15) {
+	    if (transfer[bytes_received - 1] == 0x16) {
 		transfer[bytes_received - 1] = 0;
 		fwrite(transfer, sizeof(char), bytes_received - 1, file);
 		break;
 	    }
 	    fwrite(transfer, sizeof(char), bytes_received, file);
 	}
-	child_busy_flag = 1;
 	fclose(file);
 	return (YES);
     }
@@ -637,10 +485,11 @@ int b_dp_consult(int arglist, int rest, int th)
     ind = makeind("dp_consult", n, th);
     if (n == 1) {
 	arg1 = car(arglist);
+	arg1 = makeatom(prolog_file_name(GET_NAME(arg1)), SIMP);
 	if (!atomp(arg1))
 	    exception(NOT_ATOM, ind, arg1, th);
 
-	pred1 = list2(makeatom("reconsult", SYS), arg1);
+	pred1 = list2(makeatom("consult", SYS), arg1);
 	prove_all(pred1, sp[th], th);
 
 	if (parent_flag) {
@@ -656,22 +505,24 @@ int b_dp_consult(int arglist, int rest, int th)
     return (NO);
 }
 
-int b_dp_compile(int arglist, int rest, int th)
+
+int b_dp_reconsult(int arglist, int rest, int th)
 {
     int n, ind, arg1, pred1, pred2, i;
 
     n = length(arglist);
-    ind = makeind("dp_compile", n, th);
+    ind = makeind("dp_reconsult", n, th);
     if (n == 1) {
 	arg1 = car(arglist);
+	arg1 = makeatom(prolog_file_name(GET_NAME(arg1)), SIMP);
 	if (!atomp(arg1))
 	    exception(NOT_ATOM, ind, arg1, th);
 
-	pred1 = list2(makeatom("compile_file", PRED), arg1);
+	pred1 = list2(makeatom("reconsult", SYS), arg1);
 	prove_all(pred1, sp[th], th);
 
 	if (parent_flag) {
-	    pred2 = list2(makeatom("dp_compile", SYS), arg1);
+	    pred2 = list2(makeatom("dp_reconsult", SYS), arg1);
 	    for (i = 0; i < child_num; i++) {
 		send_to_child(i, pred_to_str(pred2));
 		receive_from_child(i);
@@ -684,30 +535,47 @@ int b_dp_compile(int arglist, int rest, int th)
 }
 
 
-int b_dp_report(int arglist, int rest, int th)
+int b_dp_compile(int arglist, int rest, int th)
 {
-    int n, ind, arg1;
-    char sub_buffer[STRSIZE];
+    int n, ind, arg1, pred1, pred2, i;
 
     n = length(arglist);
-    ind = makeind("dp_report", n, th);
+    ind = makeind("dp_compile", n, th);
     if (n == 1) {
 	arg1 = car(arglist);
-	if (!stringp(arg1))
-	    exception(NOT_STR, ind, arg1, th);
+	arg1 = makeatom(prolog_file_name(GET_NAME(arg1)), SIMP);
+	if (!atomp(arg1))
+	    exception(NOT_ATOM, ind, arg1, th);
 
-	memset(sub_buffer, 0, sizeof(sub_buffer));
-	sprintf(sub_buffer, "\x02%s\x03", GET_NAME(arg1));
-	send_to_parent(makestr(sub_buffer));
-	return (prove_all(rest, sp[th], th));
+	pred1 = list2(makeatom("compile_file", PRED), arg1);
+	prove_all(pred1, sp[th], th);
+
+	if (parent_flag) {
+	    pred2 = list3(AND ,list2(makeatom("use_module",SYS),makeconst("compiler")),
+		                   list2(makeatom("compile_file", PRED), arg1));
+	    for (i = 0; i < child_num; i++) {
+		send_to_child(i, pred_to_str(pred2));
+		receive_from_child(i);
+	    }
+	}
+	return (YES);
     }
     exception(ARITY_ERR, ind, arglist, th);
     return (NO);
 }
 
+
+int all_received(int *result, int size) {
+    for (int i = 0; i < size; i++) {
+        if (result[i] == 0)  
+            return 0;   
+    }
+    return 1;             
+}
+
 int b_dp_and(int arglist, int rest, int th)
 {
-    int n, ind, arg1, m, i, j, pred, res;
+    int n, ind, arg1, m, i, j, pred, res , result[PARASIZE];
 
     n = length(arglist);
     ind = makeind("dp_and", n, th);
@@ -724,16 +592,34 @@ int b_dp_and(int arglist, int rest, int th)
 	    arg1 = cdr(arg1);
 	    i++;
 	}
-	for (i = 0; i < m; i++) {
-	    res =
-		convert_to_variant(str_to_pred(receive_from_child(i)), th);
-	    if (prove_all(res, sp[th], th) == NO) {
-		for (j = i; j < m; j++) {
-		    res =
-			convert_to_variant(str_to_pred
-					   (receive_from_child(j)), th);
+
+	for(i=0;i<m;i++){
+		result[i] = 0;
+		memset(parent_buffer[i],0,sizeof(parent_buffer[i]));
+	}
+	
+	while(!all_received(result,m)){
+		if(ctrl_c_flag == 1){
+			for(i=0;i<m;i++){
+				if(result[i] == 0)
+				send_to_child_control(i,0x11);
+			}
+			printf("ctrl+C\n");
+	    	longjmp(buf, 1);
 		}
-		return (NO);
+		for (i = 0; i < m; i++) {
+			if(parent_buffer[i][0] != 0 && result[i] == 0){
+				result[i] = 1;
+				res = convert_to_variant(str_to_pred(receive_from_child(i)), th);
+				if (prove_all(res, sp[th], th) == NO){
+					for(j=0;j<m;j++){
+						if(result[j] == 0){
+							send_to_child_control(j,0x11); // stop signal
+						}
+					}
+					return(NO);
+	            }
+		   }
 	    }
 	}
 	return (prove_all(rest, sp[th], th));
@@ -744,10 +630,10 @@ int b_dp_and(int arglist, int rest, int th)
 
 int b_dp_or(int arglist, int rest, int th)
 {
-    int n, ind, arg1, m, i, pred, res;
+    int n, ind, arg1, m, i, j, pred, res, result[PARASIZE];
 
     n = length(arglist);
-    ind = makeind("do_or", n, th);
+    ind = makeind("dp_or", n, th);
     if (n == 1) {
 	arg1 = car(arglist);
 	m = length(arg1);
@@ -761,10 +647,37 @@ int b_dp_or(int arglist, int rest, int th)
 	    arg1 = cdr(arg1);
 	    i++;
 	}
-	res =
-	    convert_to_variant(str_to_pred(receive_from_child_or(m)), th);
-	if (prove_all(res, sp[th], th) == YES)
-	    return (prove_all(rest, sp[th], th));
+
+	for(i=0;i<m;i++){
+		result[i] = 0;
+		memset(parent_buffer[i],0,sizeof(parent_buffer[i]));
+	}
+	
+	while(!all_received(result,m)){
+		if(ctrl_c_flag == 1){
+			for(i=0;i<m;i++){
+				if(result[i] == 0)
+				send_to_child_control(i,0x11);
+			}
+			printf("ctrl+C\n");
+	    	longjmp(buf, 1);
+		}
+		for (i = 0; i < m; i++) {
+			if(parent_buffer[i][0] != 0 && result[i] == 0){
+				result[i] = 1;
+				res = convert_to_variant(str_to_pred(receive_from_child(i)), th);
+				if(prove_all(res, sp[th], th) == YES){
+					for(j=0;j<m;j++){
+						if(result[j] == 0){
+							send_to_child_control(j,0x11); // stop signal
+						}
+					}
+					return (prove_all(rest, sp[th], th));
+				} 
+			}
+		}
+	}
+	return(NO);
     }
     exception(ARITY_ERR, ind, arglist, th);
     return (NO);
@@ -786,113 +699,6 @@ int b_dp_countup(int arglist, int rest, int th)
     return (NO);
 }
 
-int b_dp_parent(int arglist, int rest, int th)
-{
-    int n, ind;
-
-    n = length(arglist);
-    ind = makeind("dp_parent", n, th);
-    if (n == th) {
-
-	if (parent_flag)
-	    return (prove_all(rest, sp[th], th));
-	else
-	    return (NO);
-    }
-    exception(ARITY_ERR, ind, arglist, th);
-    return (NO);
-}
-
-int b_dp_child(int arglist, int rest, int th)
-{
-    int n, ind;
-
-    n = length(arglist);
-    ind = makeind("dp_child", n, th);
-    if (n == th) {
-
-	if (!parent_flag && child_flag)
-	    return (prove_all(rest, sp[th], th));
-	else
-	    return (NO);
-    }
-    exception(ARITY_ERR, ind, arglist, th);
-    return (NO);
-}
-
-int b_dp_wait(int arglist, int rest, int th)
-{
-    int n, ind, arg1;
-
-    n = length(arglist);
-    ind = makeind("dp_wait", n, th);
-    if (n == 1) {
-	arg1 = car(arglist);
-	if (!integerp(arg1))
-	    exception(NOT_INT, ind, arg1, th);
-	if (negativep(arg1))
-	    exception(LESS_THAN_ZERO, ind, arg1, th);
-	if (GET_INT(arg1) > 60)
-	    exception(RESOURCE_ERR, ind, makestr("over 60"), th);
-
-	sleep(GET_INT(arg1));
-	return (prove_all(rest, sp[th], th));
-    }
-    exception(ARITY_ERR, ind, arglist, th);
-    return (NO);
-}
-
-int b_dp_pause(int arglist, int rest, int th)
-{
-    int n, ind, arg1;
-    char sub_buffer[256];
-
-    n = length(arglist);
-    ind = makeind("dp_pause", n, th);
-    if (n == 1) {
-	arg1 = car(arglist);
-	if (!integerp(arg1))
-	    exception(NOT_INT, ind, arg1, th);
-	if (negativep(arg1))
-	    exception(LESS_THAN_ZERO, ind, arg1, th);
-	if (GET_INT(arg1) >= child_num)
-	    exception(RESOURCE_ERR, ind, makestr("child_num"), th);
-
-
-	memset(sub_buffer, 0, sizeof(sub_buffer));
-	sub_buffer[0] = 0x11;
-	send_to_child(GET_INT(arg1), makestr(sub_buffer));
-	return (prove_all(rest, sp[th], th));
-    }
-    exception(ARITY_ERR, ind, arglist, th);
-    return (NO);
-}
-
-int b_dp_resume(int arglist, int rest, int th)
-{
-    int n, ind, arg1;
-    char sub_buffer[256];
-
-    n = length(arglist);
-    ind = makeind("dp_resume", n, th);
-    if (n == 1) {
-	arg1 = car(arglist);
-	if (!integerp(arg1))
-	    exception(NOT_INT, ind, arg1, th);
-	if (negativep(arg1))
-	    exception(LESS_THAN_ZERO, ind, arg1, th);
-	if (GET_INT(arg1) >= child_num)
-	    exception(RESOURCE_ERR, ind, makestr("child_num"), th);
-
-	memset(sub_buffer, 0, sizeof(sub_buffer));
-	sub_buffer[0] = 0x11;
-	send_to_child(GET_INT(arg1), makestr(sub_buffer));
-	return (prove_all(rest, sp[th], th));
-    }
-    exception(ARITY_ERR, ind, arglist, th);
-    return (NO);
-}
-
 
 // multi thread parallel functions 
 void mt_enqueue(int n)
@@ -1156,3 +962,152 @@ int b_mt_prove(int arglist, int rest, int th)
     exception(ARITY_ERR, ind, arglist, th);
     return (NO);
 }
+
+void print_ascii(char *str) {
+	int i;
+    for (i = 0; i < strlen(str); i++) {
+        printf("0x%02X ", (unsigned char)str[i]);
+    }
+    printf("\n");
+}
+
+
+// Thread for parent receiver
+void *preceiver(void *arg)
+{
+	int n, m, i;
+	char buffer[BUFSIZE],sub_buffer[BUFSIZE];
+
+	n = *(int *) arg;
+
+    while (1) {
+
+	if (receiver_exit_flag)
+	    break;
+
+	// read message from parent
+	memset(buffer, 0, sizeof(buffer));
+	reread:
+	memset(sub_buffer, 0, sizeof(sub_buffer));
+    m = read(child_sockfd[n], sub_buffer, sizeof(sub_buffer));
+    if (m < 0) {
+	exception(SYSTEM_ERR, makestr("receive from child"), makeint(n),
+		  0);
+    }
+
+	//print_ascii(sub_buffer);printf("m=%d",m);fflush(stdout);
+	strcat(buffer,sub_buffer);
+
+	// normal message  0x16 is at m-1
+	if(sub_buffer[m-1] != 0x16)
+		goto reread;
+
+	//print_ascii(buffer); 
+	
+	i = strlen(buffer);
+	buffer[i] = 0;
+	strcpy(parent_buffer[n],buffer); 
+    }
+
+    pthread_exit(NULL);
+}
+
+// Thread for child receiver
+void *creceiver(void *arg)
+{
+    int n, m, i, j;
+	char buffer[BUFSIZE],sub_buffer[BUFSIZE];
+
+	if (!connect_flag) {
+	//wait conneting
+	listen(parent_sockfd[0], 5);
+	parent_len = sizeof(parent_addr);
+	connect_flag = 1;
+
+	// connection from parent
+	parent_sockfd[1] =
+	    accept(parent_sockfd[0], (struct sockaddr *) &parent_addr,
+		   &parent_len);
+	if (parent_sockfd[1] < 0) {
+	    exception(SYSTEM_ERR, makestr("*creceiver"), NIL, 0);
+	}
+    }
+
+    
+    while (1) {
+
+	if (receiver_exit_flag)
+	    break;
+
+	
+	// read message from parent
+	memset(buffer, 0, sizeof(buffer));
+	reread:
+	memset(sub_buffer, 0, sizeof(sub_buffer));
+	n = read(parent_sockfd[1], sub_buffer, sizeof(sub_buffer));
+	if (n < 0) {
+	    exception(SYSTEM_ERR, makestr("*creceiver"), NIL, 0);
+	}
+
+	strcat(buffer,sub_buffer);
+	if(sub_buffer[n-1] != 0x16)
+		goto reread;
+
+	pthread_mutex_lock(&mutex2);
+
+	
+	j = 0;
+	
+	
+	m = strlen(buffer);
+
+	for (i = 0; i < m-1; i++) {
+		if(buffer[i] == 0x11){
+			memset(child_buffer,0,sizeof(child_buffer));
+			strcpy(child_buffer,"fail.");
+			child_buffer_pos = 0;
+			child_buffer_end = 5;
+			ctrl_c_flag = 1;
+			goto exit;
+		}
+	    else { 
+			child_buffer[j] = buffer[i];
+	    	j++;
+		}
+	}
+
+	child_buffer_pos = 0;
+	child_buffer_end = j;
+
+	exit:
+	printf("receive from parent %s\n",child_buffer);
+
+	child_buffer_ready = 1;
+	pthread_cond_signal(&md_cond);
+	pthread_mutex_unlock(&mutex2);
+
+	
+    }
+
+    pthread_exit(NULL);
+}
+
+
+void init_preceiver(int n)
+{	
+	int i;
+    // create parent receiver thread 
+	for(i=0;i<n;i++){
+		int *arg = malloc(sizeof(int));
+        *arg = i;
+    	pthread_create(&preceiver_thread[i], NULL, preceiver, arg);
+	}
+
+}
+
+void init_creceiver(void)
+{
+    // create child receiver thread 
+    pthread_create(&creceiver_thread, NULL, creceiver, NULL);
+
+}
