@@ -98,10 +98,10 @@ char extended[EXTENDED_NUMBER][30] = {
     { "pwm_write" }, { "pull_up_dn_control" }, { "digital_read" },
     { "delay" }, { "delay_microseconds" },
     { "compile_file" }, { "timer_microseconds" }, { "with" },
-    { "existerrors" }, { "dp_create" }, { "dp_consult" },
-    { "dp_transfer" }, { "dp_and" }, { "dp_or" },
-    { "dp_compile" }, { "dp_prove" }, { "dp_close" }, { "dp_parent" },
-    { "dp_child" }, { "dp_wait" }, { "dp_pause" }, { "dp_resume" },
+    { "existerrors" }, { "dp_create" }, { "dp_consult" }, { "dp_countup" },
+    { "dp_transfer" }, { "dp_and" }, { "dp_or" }, { "dp_halt" },
+    { "dp_receive" }, { "dp_reconsult" }, { "dp_setid" }, { "dp_senderr" },
+    { "dp_compile" }, { "dp_prove" }, { "dp_close" }, 
     { "mt_create" }, { "mt_close" }, { "mt_and" }, { "mt_or" },
     { "mt_prove" }, { "cinline" }, { "check_file" }, { "cdeclare" },
     { "clibrary" },
